c collprob_follow.f
c Calculate intrinsic collisional probabilities for an AstOrb-like file
c   and a swift binary file (integer,real,real*8).
c Miroslav Broz, miroslav.broz@email.cz, Mar 24th 2011

      program follow2

c-----------------------------------------------------------------------
      implicit none

      integer nmax,proba_max
      parameter(nmax=30000)       ! maximum number of bodies (orbital elements) in each input file
      parameter(proba_max=10000000)  ! maximum number of orbit PAIRS for which we calculate probabilities

c constants
      real*8 pi,deg,rad
      parameter(pi=3.1415926535d0, deg=pi/180.d0, rad=180.d0/pi)

c input params
      character*80 infile1, infile2
      integer ftype, n2_min, n2_max, idum
      real*8 t_unit, q_crit, p_crit, v_esc
      logical out_orb

c temp vars
      real*8 a1(nmax),a2(nmax),e1(nmax),e2(nmax),inc1(nmax),inc2(nmax)
      integer ids(nmax),istat(nmax)
      real*8 proba,relvel,rmsvel,impvel,AU,q,p
      integer n1,n2,numint_cap,numint_the,n_pairs
      integer i,j,k,l,id,istep,ierr,nbod,nleft
      real*8 t,tmax,a,e,inc,capom,omega,capm
      real*8 proba_arr(proba_max), proba_mean, proba_sigma
      real*8 impvel_arr(proba_max), impvel_mean, impvel_sigma
      character*80 str,outfile
      logical crossing

c functions
      real*8 ran1
      integer io_read_hdr,io_read_line
      integer io_read_hdr_r,io_read_line_r
      integer io_read_hdr_r8,io_read_line_r8

c-----------------------------------------------------------------------

c read input params from stdin
      read(*,5,err=990,end=990) infile1
5     format(a)
      read(*,5,err=990,end=990) infile2
      read(*,*,err=990,end=990) ftype
      read(*,*,err=990,end=990) n2_min, n2_max
      read(*,*,err=990,end=990) t_unit
      read(*,*,err=990,end=990) out_orb
      read(*,*,err=990,end=990) q_crit
      read(*,*,err=990,end=990) p_crit
      read(*,*,err=990,end=990) idum
      read(*,*,err=990,end=990) v_esc

c write parameter to stdout
      write(*,*) '# n2_min = ', n2_min
      write(*,*) '# n2_max = ', n2_max
      write(*,*) '# q_crit = ', q_crit, ' AU'
      write(*,*) '# p_crit = ', p_crit

c check file type
      if (ftype.eq.2) then
      else if (ftype.eq.1) then
      else if (ftype.eq.0) then
      else
        write(*,*) 'follow2: Unknown binary file format specified.'
        stop
      endif

c initialize istat array
      do i = 1, nmax
        istat(i) = 0
      enddo

c read astorb-like file
      n1 = nmax
      call read_astorb(infile1,a1,e1,inc1,n1)

      AU = 1.4959787d13        ! in centimeters!
      do i = 1, n1
        a1(i) = a1(i)*AU
        inc1(i) = inc1(i)*deg  ! inclination from astorb is in degrees!
      enddo

c write header
      write(*,*) '# Binary input filename: ', infile2
      write(*,40)
40    format(' # t [Myr] & proba [10^-18 km^-2 yr^-1] mean & sigma',
     :  ' & impvel [km/s] mean & sigma & n1 & n2 & n_proba & n_impvel')

c read binary file
      open(unit=1,file=infile2,status='old',form='unformatted',
     :  iostat=ierr)
      if (ierr.ne.0) then
        write(*,*) 'follow2: Error opening file ',infile2
        stop
      endif

      if (out_orb) then
        outfile = "collprob_follow_q.tmp"
        open(unit=2,file=outfile,status='unknown',iostat=ierr)
        if (ierr.ne.0) then
          write(*,*) 'follow2: Error opening file ', outfile
          stop
        endif
      endif

      tmax = 0.d0
10    continue
        j = 0
        istep = 0
        if (ftype.eq.2) then
          ierr = io_read_hdr_r8(1,t,nbod,nleft)
        else if (ftype.eq.1) then
          ierr = io_read_hdr_r(1,t,nbod,nleft)
        else if (ftype.eq.0) then
          ierr = io_read_hdr(1,t,nbod,nleft)
        endif
        if (ierr.ne.0) goto 20
        t = t/t_unit	! Myr
        do i=2,nbod
          if (ftype.eq.2) then
            ierr = io_read_line_r8(1,id,a,e,inc,capom,omega,capm)
          else if (ftype.eq.1) then
            ierr = io_read_line_r(1,id,a,e,inc,capom,omega,capm)
          else if (ftype.eq.0) then
            ierr = io_read_line(1,id,a,e,inc,capom,omega,capm)
          endif
          if (ierr.ne.0) goto 20
          istep = 1
          tmax = t
          j = j + 1
          ids(j) = id
          a2(j) = a
          e2(j) = e
          inc2(j) = inc
        enddo

        do i=1,nleft
          if (ftype.eq.2) then
            ierr = io_read_line_r8(1,id,a,e,inc,capom,omega,capm)
          else if (ftype.eq.1) then
            ierr = io_read_line_r(1,id,a,e,inc,capom,omega,capm)
          else if (ftype.eq.0) then
            ierr = io_read_line(1,id,a,e,inc,capom,omega,capm)
          endif
          if (ierr.ne.0) goto 20
          istep = 1
          tmax = t
          j = j + 1
          ids(j) = id
          a2(j) = a
          e2(j) = e
          inc2(j) = inc
        enddo
        if (istep.eq.0) goto 20
        n2 = j

c calculate collisional probabilities and impact velocities
        numint_cap = 10
        numint_the = 10
c        v_esc = 0.d0      ! km/s -> no focusing in case of asteroids!

        n_pairs = n1 * (min(n2, n2_max) - n2_min)
        if (n_pairs.gt.proba_max) then
          write(*,*) 'Error: number of orbit pairs ', n_pairs,
     :      ' larger than proba_max = ', proba_max
          stop
        endif

        do j = 1, n2
          a2(j) = a2(j)*AU   ! convert units to cm
        enddo

        k = 0
        l = 0
        do j = n2_min, min(n2, n2_max)
          if (istat(ids(j)).eq.0) then

            crossing = .false.
            do i = 1, n1

              relvel = 0.d0
              rmsvel = 0.d0
              impvel = 0.d0

              call gaussorb(a1(i),e1(i),inc1(i), a2(j),e2(j),inc2(j),
     :          proba,relvel,rmsvel,impvel,numint_cap,numint_the,v_esc)

              k = k + 1
              proba_arr(k) = proba
              impvel_arr(k) = impvel
              if (impvel.gt.1.d-8) then
                l = l + 1
                crossing = .true.
              endif

            enddo

            if ((out_orb).and.(crossing)) then
              write(2,50) ids(j), t, a2(j)/AU, e2(j), inc2(j)
50            format(1x,i8,1x,f16.10,1x,f14.8,1x,f10.8,1(1x,f10.5))
            endif

c comet-disruption criterion
            q = a2(j)/AU * (1.d0-e2(j))
            if (q.lt.q_crit) then
              p = ran1(idum)
              if (p.le.p_crit) then
                istat(ids(j)) = 1
              endif
            endif

          endif
        enddo

        call mean_sigma(proba_arr, k, proba_mean, proba_sigma)
        call mean_sigma_weight(impvel_arr, proba_arr, k, impvel_mean,
     :    impvel_sigma)

        write(*,30) t, proba_mean, proba_sigma, impvel_mean,
     :    impvel_sigma, n1, n2, k, l
30      format(f16.10,4(1x,f22.16),4(1x,i8))

      goto 10

20    continue
      close(1)
      write(*,*) '# follow2: t_max = ',tmax,' Myr'

      if (out_orb) then
        close(2)
      endif

      stop

c error handlers

990   continue
      write(*,*) 'Error reading standard input.'
      stop

      end

