
C**********************************************************************
      subroutine c_equation (a, a0, e, e0, c, c0)
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  This subroutine solves C or Co, depending on the input variables
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
      implicit NONE 
c
      real * 8  a, a0 
      real * 8  e, e0 
      real * 8  c, c0 
      real * 8  p, p0, alphay0
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
ccc
c  The semilatus rectum for the test body: p
ccc
      p = a * (1 - e * e) 
ccc
c  The semilatus rectum for the field body: p0
ccc
      p0 = a0 * (1 - e0 * e0) 
ccc
c  Setting up C = cos (u - w) with Co = cos (uo - wo) 
ccc
      alphay0 = 1 + e0 * c0
c
      c = (alphay0 * p / p0 - 1) / e
c
      return
      end
      subroutine gauleg (x1, x2, x, w, n)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  Program found in: "Numerical Recipes", Chapter 4.5
c
c  Given the lower (x1) and upper (x2) limits of integration, this
c  routine returns arrays (x) and (w) of length (n), containing the
c  abscissas and weights of the Gauss-Legendre N-point quadrature
c  formula.
c
c  The recurrence relation for Gauss-Legendre is {(-1, 1), W (x) = 1}:
c
c    (i + 1) P        = (2i + 1)(x) P  - (i) P 
c             (i + 1)                i        (i - 1)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
ccc
c  High precision is a good idea for this routine:
ccc
      implicit double precision (a-h, o-z)
c
      real * 8 x1, x2
      real * 8 x (5000), w (5000)
ccc
c  Increase of you do not have floating precision
ccc
      parameter (eps = 3d-14)
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
ccc
c  The roots are symmetric in the interval, so we only have to find half
c  of them.
ccc
      m = (n + 1) / 2
      xm = 0.5d0 * (x2 + x1)
      xl = 0.5d0 * (x2 - x1)
ccc
c  Loop over the desired roots
ccc
      do 300 i = 1, m
         z = dcos (3.141592654d0 
     $       * (dble (i) - .25d0) / (dble (n) + .5d0))
ccc
c  Starting with the above approximation to the i'th root, we enter
c  the main loop of refinement by Newton's method.
ccc
  100    continue
            p1 = 1d0
            p2 = 0d0
ccc
c  Loop up the recurrence relation to get the Legendre polynomial
c  evaluated at (z).  
c
c  The recurrence relation for Gauss-Legendre is:
c
c    (i + 1) P        = (2i + 1)(x) P  - (i) P 
c             (i + 1)                i        (i - 1)
ccc
            do 200 j = 1, n
               p3 = p2
               p2 = p1
               p1 = ((2d0 * dble(j) - 1d0) * z * p2
     $	            - (dble (j) -1d0) * p3) / dble (j)
  200       continue
ccc
c  (p1) is now the desired Legendre polynomial.  We next compute (pp),
c  its derivative, by a standard relation involving also (p2), the
c  polynomial of one lower order.
ccc
            pp = dble (n) * (z * p1 - p2) / (z * z - 1d0)
            z1 = z
ccc
c  Newton's method
ccc
            z = z1 - p1 / pp
         if (dabs (z - z1) .gt. eps) go to 100
ccc
c  Scale the root to the desired interval...
ccc
         x (i) = xm - xl * z
ccc
c  ...and put in its symmetric counterpart.
ccc
         x (n + 1 - i) = xm + xl * z
ccc
c  Compute the weight...
c
c  The weight for the Gaussian formula for Legendre polynomials is:
c
c                         2
c      w    =  ------------------------- 
c       i      (1 - x ^ 2) [ P' (x )] ^2 
c                     i        N   i
ccc
         w (i) = 2d0 * xl / ((1d0 - z * z) * pp * pp)
ccc
c  ...and its symmetric counterpart.
ccc
         w (n + 1 - i) = w (i)
  300 continue
c
      return
      end
      subroutine gaussorb  (a, e, i, a0, e0, i0,
     $                      finalcoll, relvel, rmsvel, impvel, 
     $                      numint_cap, numint_the, v_esc)
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  This program calculates the collison probability of a chosen
c  field and test body given their orbital parameters.  It also can
c  calculate the mean relavtive velocity as well as the rms
c  velocity.
c
c  Last Modification : April 28, 1992
c
c  Based on:  Greenberg, R., (1982)  "Orbital interactions: A new
c             geometrical formalism.", Astron. Jour., v. 87
c             pp. 184--195.
c
c
c  Written by:  Bill Bottke
c               LPL / University of Arizona
c               Tucson, AZ 85721
c               (602) 621 - 4655
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
      implicit NONE
c
      integer numint_cap, numint_the
      integer k
c
      real * 8 cmin, cmax, cbottom, ctop, c0min, c0max
      real * 8 cmintest, cmaxtest
      real * 8 a0, e0, i0, a, e, i
      real * 8 ax1, ay1, ex1, ey1, ix1, iy1
      real * 8 ax2, ay2, ex2, ey2, ix2, iy2
      real * 8 cxmin1, cxmax1, cxmin2, cxmax2
      real * 8 xomgmin, xomgmax
      real * 8 xconst, precoll, finalcoll, relvel
      real * 8 rmsvel, impvel 
      real * 8 v_esc
c
      real * 8 coll (4)
      real * 8 coll2 (4)
      real * 8 colltot (4)
c
      logical linetest
c
      include 'g_constant.inc'
      include 'g_xyzconst.inc'
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
ccc
c  Parameters
ccc
      cbottom = -1.0
      ctop = 1.0
c
      cmin = 0d0
      cmax = 0d0 
      c0min = 0d0
      c0max = 0d0 
ccc
c  Check limits of C0 first: Perform a check to find min and max C
c
c     Note:  These tests are rather "longwinded" and it is very easy to 
c            simplify it.  However, it yields the correct results
c            for the min and max of C and C0 so it is not really necessary
c            to make it more elegant.
ccc
      call c_equation (a, a0, e, e0, cmintest, cbottom)
      if (cmintest .ge. -1.0 .and. cmintest .le. 1.0) then
	 cmin = cmintest
	 c0min = cbottom 
      endif
c
      call c_equation (a, a0, e, e0, cmaxtest, ctop)
      if (cmaxtest .ge. -1.0 .and. cmaxtest .le. 1.0) then
	 cmax = cmaxtest
	 c0max = ctop 
      endif
ccc
c  Check limits of C second: Perform a check to find min and max C0
ccc
      call c_equation (a0, a, e0, e, cmintest, cbottom)
      if (cmintest .ge. -1.0 .and. cmintest .le. 1.0) then
	 c0min = cmintest
	 cmin = cbottom 
      endif
c
      call c_equation (a0, a, e0, e, cmaxtest, ctop)
      if (cmaxtest .ge. -1.0 .and. cmaxtest .le. 1.0) then
	 c0max = cmaxtest
	 cmax = ctop 
      endif
c
      if ((cmin  .ne. -1.0 .and. cmax  .ne. 1.0) .and. 
     $    (c0min .ne. -1.0 .and. c0max .ne. 1.0)) then
c	 write (6, *) 'No C, C0 box intersection'
	 finalcoll = 0d0
         return
      endif
c
      call linecheck (cmin, cmax, c0min, c0max,
     $                a, a0, e, e0, i, i0,
     $                ax1, ay1, ex1, ey1, ix1, iy1,  
     $                cxmin1, cxmax1, 
     $                ax2, ay2, ex2, ey2, ix2, iy2, 
     $                cxmin2, cxmax2,
     $                linetest)
ccc
c  Parameters for the integration:
ccc
      do k = 1, 4 
         colltot (k) = 0d0
	 coll (k)    = 0d0
	 coll2 (k)   = 0d0
      enddo
c
      xomgmin = 0.0
      xomgmax = pi 
ccc
c  1)  Perform the collision probability integral:
c
c  2)  Calculate the mean relative encounter velocity by putting velocity 
c      into the collision integral, integrating, and dividing by
c      the collision probability without velocity.  Don't forget to put
c      the appropriate units in for the velocity at the end {km/s}.
c
c  3)  Calculate the RMS relative encounter velocity by putting {Vi^2}
c      into the collision integral, integrating, and dividing by
c      the collision probability without velocity.  Don't forget to put
c      the appropriate units in for the velocity at the end {km/s}.
ccc
ccc
c  Integration of two parts to avoid singularity: 
ccc
      if (linetest) then
	 call g_integral (ax1, ay1, ex1, ey1, ix1, iy1, 
     $                    xomgmin, xomgmax, cxmin1, cxmax1, coll,
     $                    numint_cap, numint_the, v_esc)

c
         call g_integral (ax2, ay2, ex2, ey2, ix2, iy2, 
     $                    xomgmin, xomgmax, cxmin2, cxmax2, coll2,
     $                    numint_cap, numint_the, v_esc)
         do k = 1, 4 
            colltot (k) = coll (k) + coll2 (k)
         enddo
      else
ccc
c  Integratation of whole: 
ccc
         call g_integral (ax1, ay1, ex1, ey1, ix1, iy1, 
     $                    xomgmin, xomgmax, cxmin1, cxmax2, colltot,
     $                    numint_cap, numint_the, v_esc)
      endif
ccc
c  Calculate the collision prob:  The 2 takes into account
c  the integration from {0, pi} rather than {0, 2pi} for capital
c  omega.  Note that the 4 takes into account both sides of the 
c  Dmin = 0 line as well as the interactions at the other mutual node. 
ccc
      xconst = 2. * 4. / (tau * tau * (2 * pi) ** (3.0))
      precoll = xconst * colltot (1) 
ccc
c  We need to put the collision probability into units 
c  of 1.0 e-18 km-2 yr-1.
c
c  To do this, we divide by (1 / 2pi) for the period.
ccc
      finalcoll = precoll * (86400d0 * 365d0 * 1.0d10)
      finalcoll = finalcoll / 1.0d-18
ccc
c  Calculate the rel. collision velocity: The 2 takes into account
c  the integration from {0, pi} rather than {0, 2pi} for capital
c  omega.  Note that the 4 takes into account both sides of the 
c  Dmin = 0 line as well as the interactions at the other mutual node. 
c  We also divide by the collision probability to get the mean 
c  relative encounter velocity.
c
c  We need to put the relative collision velocity into units 
c  of km sec-1.
ccc
      relvel = xconst * colltot (2) / 1.0d5 / precoll  
ccc
c  Calculate the RMS collision velocity:  "   "    "    "    "
c  We also divide by the collision probability to get the mean 
c  relative encounter velocity.  The square root is due to the V^2 
c  power in the integral
ccc
      rmsvel = dsqrt (xconst * colltot (3) / precoll) / 1.0d5 
ccc
c  Calculate the impact velocity:  "   "    "    "    "
c  We also divide by the collision probability to get the mean 
c  relative encounter velocity.  The square root is due to the V^2 
c  power in the integral
ccc
      impvel = xconst * colltot (4) / precoll 
c
      return
      end
      subroutine g_integral (a, a0, e, e0, i, i0, 
     $                       capmin, capmax, cmin, cmax, coll,
     $                       numint_cap, numint_the, v_esc)
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  This subroutine solves the guts of the integral.  Using the
c  parameters, the program returns the value of the collision
c  probability, the relative velocity, and the rms velocity
c  for final normalization.
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
      implicit NONE 
c
      integer numint_cap, numint_the
      integer jcap, jtheta 
      integer irelvel, irel_r, irel_t, irel_z
      integer kk
c
      real * 8 a, a0, e, e0, i, i0
      real * 8 capmin, capmax, cmin, cmax
      real * 8 ipcos, ipsin, ipsinsq
      real * 8 mu 
      real * 8 thetamin, thetamax, sub
      real * 8 coll_units, ci, ci0, si, si0
      real * 8 p, p0, period, period0 
      real * 8 delomg, sd, cd, t11, t12, t13, t21, t22
      real * 8 t23, t31, t32, uwsign0, uwsign, theta0
      real * 8 c0, alphay0, c, alphay, theta 
      real * 8 cotalpha, cotalpha0, cotasq, cotasq0
      real * 8 capa, derivdmin, delcapc, alphax0, alphaz0
      real * 8 alphasin0, alphacos0, u0, ux0, uy0, uz0
      real * 8 alphax, alphaz, alphasin, alphacos, u, ux
      real * 8 uy, uz, v_x, v_y, v_z
      real * 8 vel, bottometa0, eta0, capp0, partcoll, probbin
      real * 8 vx, vy, vz, r_x, rx, ry, rz, v_plane
      real * 8 theta_v, theta_r, phi, v_r, v_theta
      real * 8 relvel, rel_r, rel_t, rel_z
      real * 8 v_esc, focus
c
      real * 8 coll (4) 
c
      real * 8 xcap (5000)
      real * 8 wcap (5000)
c
      real * 8 xtheta (5000)
      real * 8 wtheta (5000)
c
      include 'g_constant.inc'
      include 'g_xyzconst.inc'
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
ccc
c  Here we find the limits of the true anomaly (theta):
ccc
      thetamin = dacos (cmin)
      thetamax = dacos (cmax)
      if (thetamin .gt. thetamax) then
         sub = thetamin
         thetamin = thetamax
         thetamax = sub
      endif
ccc
c  The mass parameter (mu):
ccc
      mu = g * msun
ccc
c  Place the collition probability for this particular relative velocity 
c  into units used by the total collision probabilty: 
c
c  We need to put the collision probabiltiy into units
c  of 1.0 e-18 km-2 yr-1.
ccc
      coll_units = 2 * 4 / (tau * tau * (2 * pi) ** (3.0))
     $             * (86400d0 * 365d0 * 1.0d10) / 1.0d-18
ccc
c  Set of the sine and cosines for many of the variables
ccc
      ci  = dcos (i)
      ci0 = dcos (i0)
      si  = dsin (i)
      si0 = dsin (i0)
ccc
c  The semilatus rectum for the test body: p
ccc
      p = a * (1 - e * e) 
ccc
c  The semilatus rectum for the field body: p0
ccc
      p0 = a0 * (1 - e0 * e0) 
ccc
c  Define T, To
ccc
      period  = 2d0 * pi * a ** three2 / dsqrt (mu)
      period0 = 2d0 * pi * a0 ** three2 / dsqrt (mu)
ccc
c  Find the abscissas and weights of the Gauss-Legendre N-point quadrature
c  formula.
ccc
      call gauleg (capmin, capmax, xcap, wcap, numint_cap)
c
      call gauleg (thetamin, thetamax, xtheta, wtheta, numint_the)
ccc
c  This loop integrates the Delta (Capital Omega) variable as a
c  function of the integral:
ccc
      do 100 jcap = 1, numint_cap
ccc
c  This subroutine takes care of the regions of the orbital intersection
c  where the sign of the true anomaly can be (+) or (-).
ccc
ccc
c  This loop is not necessary since the true anomaly solutions
c  are symmetric for (+, +), (-, -) and (+, -), (-, +).  However,
c  one can put it in by commenting out the redundant passages and
c  multiplying the integral by a factor of 2.
ccc
         delomg = xcap (jcap)
	 sd = dsin (delomg)
	 cd = dcos (delomg)
ccc
c  Define cos (i'): i  = i0, i = i
c                    o
ccc
         ipcos = ci * ci0 + si * si0 * cd
ccc
c  Define sin (i'): 
c    We choose a sign for sin i' by fining the sign for both sin (delomg)
c    and sin (upos0).  If both are between {0, pi} or {pi, 2pi} then
c    we are dealing with the acending node and sin i' is positive.  However,
c    if one is between {0, pi} and the other is between {pi, 2pi} then
c    we are dealing with the decending node and the sign is negative.
c    Essentially, whether both nodes have positive sin i' or negative sin i'.
c
c    Now, we can ignore all of this since we always choose delta (capomg)
c    to between the limits of [0, pi].  Thus, the sin (delomg) is always
c    positive.  Choosing one of the mutual node to be the ascending 
c    node (and therefore positive), we get a positive sin (i').  Therefore,
c    we always are choosing upos0 to be positive.
ccc
         ipsin = dsqrt (1d0 - ipcos * ipcos)
         ipsinsq = ipsin * ipsin
ccc
c  Thus, our rotation matrix is:
c
c                           (X)             (X'') 
c                           (Y) = {R} o {T} (Y'') 
c                           (Z)             (Z'') 
c
c  Rotation R: This rotation is with respect to the longitude of ascending
c              node of the field body (Capital Omega_o).  However, since we
c              wish to randomize this variable from (0, 2pi), we can ignore
c              this variable
c              
c
c    [  cos(i0)cos(i') - cos(i)          sin(i)sin(D)                       ]
c    [  =======================          ============                 0     ]
c    [       sin(i')sin(i0)                 sin(i')                         ]
c    [                                                                      ]
c    [    cos(i0)sin(i)sin(D)     sin(i)cos(D) - sin(i0)cos(i')             ]
cT = [ -  ===================     =============================   - sin(i0) ]
c    [          sin(i')                     sin(i')                         ]
c    [                                                                      ]
c    [    sin(i0)sin(i)sin(D)        cos(i0)cos(i') - cos(i)                ]
c    [ -  ===================        =======================        cos(i0) ]
c    [          sin(i')                     sin(i')                         ]
c
ccc
      t11 =   (-si0 * ci + ci0 * si * cd) / ipsin 
c     t11 =   (ci0 * ipcos - ci)      / (ipsin * si0) 
      t12 =    si  * sd               / ipsin
      t13 =    0d0
c
      t21 = - (ci0 * si * sd)         / ipsin 
      t22 =   (si * cd - si0 * ipcos) / ipsin 
      t23 = -  si0 
c
      t31 = -  si0 * si * sd          / ipsin 
      t32 =   (ci0 * ipcos - ci)      / ipsin 
ccc
c  Loop through the true anomaly (theta, theta0) with all of its 
c  collision values:
c
c               { theta, theta0     theta, -theta0}
c               {-theta, theta0    -theta, -theta0}
ccc
         do 200 jtheta = 1, numint_the 
            do 300 uwsign0 = -1d0, 1d0, 2d0
	    do 400 uwsign  = -1d0, 1d0, 2d0
ccc
c  Solve for c0:
ccc
                  theta0 = xtheta (jtheta)
                  c0 = dcos (theta0)
ccc
c  Setting up C = cos (u - w) with Co = cos (uo - wo) 
ccc
                  alphay0 = 1 + e0 * c0
                  c = (alphay0 * p / p0 - 1d0) / e
                  alphay  = 1 + e  * c
		  theta = dacos (c)
ccc
c  Define cot (alpha), cot (alpha0): e = e0, e0 = e 
ccc
                  cotalpha  = e * dsin (theta) / alphay 
                  cotalpha  = sign (cotalpha, uwsign) 
                  cotalpha0 = e0 * dsin (theta0) / alphay0 
                  cotalpha0 = sign (cotalpha0, uwsign0) 
ccc
c  Define A: 
ccc
                  cotasq  = cotalpha  * cotalpha 
                  cotasq0 = cotalpha0 * cotalpha0 
                  capa = dsqrt (cotasq + cotasq0 + ipsinsq 
     $                        - 2d0 * cotalpha * cotalpha0 * ipcos)
ccc
c  Define Delta C: 
ccc
                  derivdmin = abs (-1d0 * p * e * ipsin   
     $                        / (capa * alphay * alphay))
                  delcapc = tau / derivdmin
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  The Second Part of the Integral (Po, To, T, etc...)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
ccc
c  The Field Body:
ccc
ccc
c  Setting up important parameters for cos (alpha0), sin (alpha0)
ccc
                  alphax0 = e0 * dsqrt (1d0 - c0 * c0) 
                  alphax0 = sign (alphax0, uwsign0) 
c                 alphay0 = 1d0 + e0 * c0
                  alphaz0 = dsqrt (1d0 + 2d0 * e0 * c0 + e0 * e0)
c
                  alphasin0 = alphay0 / alphaz0
                  alphacos0 = alphax0 / alphaz0 
ccc
c  The velocity of the field body (Uo):
ccc
                  u0 = dsqrt (mu / p0) * alphaz0 
ccc
c  The velocities of the field body: ux0, uy0, uz0
ccc
                  ux0 = u0 * alphacos0 
                  uy0 = u0 * alphasin0 
                  uz0 = 0
ccc
c  The Test Body:
ccc
ccc
c  Setting up important parameters for cos (alpha), sin (alpha)
ccc
                  alphax = e * dsqrt (1 - c * c) 
                  alphax = sign (alphax, uwsign) 
c                 alphay = 1d0 + e * c
                  alphaz = dsqrt (1d0 + 2d0 * e * c + e * e)
c
                  alphasin = alphay / alphaz
                  alphacos = alphax / alphaz 
ccc
c  The velocity of the test body (U):
ccc
                  u = sqrt (mu / p) * alphaz 
ccc
c  The velocities of the field body: ux, uy, uz 
ccc
                  ux = u * alphacos 
                  uy = u * alphasin * ipcos 
                  uz = u * alphasin * ipsin 
ccc
c  The relative velocities: v_x, v_y, v_z, vel
ccc
                  v_x = ux0 - ux
                  v_y = uy0 - uy
                  v_z = uz0 - uz
                  vel  = dsqrt (v_x * v_x + v_y * v_y + v_z * v_z)
ccc
c  Define eta0:
ccc
                  bottometa0 = dsqrt (vel * vel
     $               - (v_x * alphacos0 + v_y * alphasin0) ** 2d0)
                  eta0 = tau * vel / bottometa0
ccc
c  Define Po:
ccc
                  capp0 = 2 * eta0 / (u0 * period0)
ccc
c  Put the integral all together:
ccc
                  partcoll = delcapc * capp0 * pi 
     $                       / (4d0 * dsqrt (1d0 - c * c) * period)
ccc
c  For planets, we need to include to gravitational focussing factor to 
c  account for "tau"^2 correctly
ccc
                  relvel = dabs (vel)      * 1.0d-5          ! V_inf [km/s]

                  if (relvel .le. 0) then 
                     focus = 0.0
                  else
                     focus = 1.d0 + v_esc * v_esc 
     $                             / (relvel * relvel)
                  endif
ccc
c  Call the binning subroutine:
ccc
		  partcoll = partcoll * wtheta (jtheta) 
     $                                * wcap (jcap) 
     $                                * focus 
		  probbin = partcoll * coll_units  
ccc
c  Determine the velocity components with respect to the invarient
c  plane of the planets.  The subroutine "rotaz" places the components
c  into that plane.
ccc
c                 vx = v_x * t11 + v_y * t12 
c                 vy = v_x * t21 + v_y * t22 - v_z * si0 
c                 vz = v_x * t31 + v_y * t32 + v_z * ci0 
ccc
c  The direction of the r vector is the direction of the line of 
c  mutual nodes (x'') and therefore positive
ccc
c                 r_x = p0 / alphay0
c                 rx = r_x * t11 
c                 ry = r_x * t21 
c                 rz = r_x * t31 
ccc
c  Find the angle (phi) between the in-plane velocity vector and the 
c  in-plane collision position vector.
ccc
c                 v_plane = dsqrt (vx * vx + vy * vy) 
c                 theta_v = datan2 (vy, vx)
c	          if (theta_v .lt. 0d0) theta_v = theta_v + 2 * pi 
c                 theta_r = datan2 (ry, rx)
c	          if (theta_r .lt. 0d0) theta_r = theta_r + 2 * pi 
ccc
c  This code to account for symmetry in phi (see notes) between (-pi, 0)
c  and (0, 2pi)
ccc
c                 phi = theta_v - theta_r
c                 if (phi .lt. -pi) phi = phi + 2 * pi
c                 if (phi .gt.  pi) phi = phi - 2 * pi
c                 phi = dabs (phi)
ccc
c  The V_r and V_theta collision velcocity components
ccc
c                 v_r     = v_plane * dcos (phi)
c                 v_theta = v_plane * dsin (phi)
ccc
c  Convert the relative velocity into units we know [km / s] 
ccc
                  relvel = dabs (vel)      * 1.0d-5 
c                 rel_r  = dabs (v_r)      * 1.0d-5 
c                 rel_t  = dabs (v_theta)  * 1.0d-5 
c                 rel_z  = dabs (vz)       * 1.0d-5 
ccc
c  Convert the velocity into a "bin size" by dividing by the 
c  bin size and finding the integer.
ccc
                  irelvel    = int (relvel / binsize) 
c                 irel_r     = int (rel_r  / binsize) 
c                 irel_t     = int (rel_t  / binsize) 
c                 irel_z     = int (rel_z  / binsize) 
ccc
c  Place the collision probability into the appropriate velocity bin: 
ccc
		  coll (1) = coll (1) + partcoll
		  coll (2) = coll (2) + partcoll * vel
		  coll (3) = coll (3) + partcoll * vel * vel
c
		  coll (4) = coll (4) + partcoll 
     $                * sqrt (relvel * relvel + v_esc * v_esc)
c
  400       continue
  300       continue
  200    continue
  100 continue
c
      return
      end
      subroutine linecheck (cmin, cmax, c0min, c0max, 
     $                      a, a0, e, e0, i, i0, 
     $                      ax1, ay1, ex1, ey1, ix1, iy1, 
     $                      cxmin1, cxmax1,
     $                      ax2, ay2, ex2, ey2, ix2, iy2,
     $                      cxmin2, cxmax2,
     $                      linetest)
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  Set the parameters such that the program can integrate in 
c  one or two parts.  Program could be simplified, but it is
c  much easier to keep track of the relative parts with a debugger 
c  if this subroutine is used.
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
      implicit NONE
c
      real * 8 cmin, cmax, c0min, c0max
      real * 8 a, a0, e, e0, i, i0
      real * 8 ax1, ay1, ex1, ey1, ix1, iy1
      real * 8 cxmin1, cxmax1
      real * 8 ax2, ay2, ex2, ey2, ix2, iy2
      real * 8 cxmin2, cxmax2
c
      logical linetest
c
      include 'g_constant.inc'
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
ccc
c  The line choices:
c                       xxxx*xxxxxxx#xxxxxxxxxxxxxxxx
c                       x  *        #               x
c                       x * line 1 #                x
c                       x*         # line 3         x
c                       *          #                x
c                       x         #             ***** 
c                C0     x         # ************    *
c                       x      *****   line 2      *x
c                       *******  #                * x
c                       x        #               *  x
c                       x        #              *   x
c                       x       #       line 4 *    x
c                       x       #             *     x 
c                       xxxxxxxx#xxxxxxxxxxxx*xxxxxxx
c
c                                      C
ccc
c Line 1:
ccc
      if (c0min .gt. -1.0 .and. c0max .eq. 1.0 .and.
     $    cmin  .eq. -1.0 .and. cmax  .lt. 1.0) then 
c
          ax1 = a0
          ex1 = e0 
          ix1 = i0 
          ay1 = a 
          ey1 = e 
          iy1 = i 
	  cxmin1 = cmin
	  cxmax1 = cmin + (cmax - cmin) / 2.0
c
          ax2 = a 
          ex2 = e 
          ix2 = i 
          ay2 = a0 
          ey2 = e0 
          iy2 = i0 
ccc
c  Get a new value for Cmin and Cmax so you can integrate the
c  rest of the way.
ccc
          call c_equation (ax1, ay1, ex1, ey1, cxmin2, cxmax1)
	  cxmax2 = c0max
	  linetest = .true.
ccc
c Line 2:
ccc
      elseif (cmin  .eq. -1.0 .and. cmax  .eq. 1.0) then
c
          ax1 = a0
          ex1 = e0 
          ix1 = i0 
          ay1 = a 
          ey1 = e 
          iy1 = i 
	  cxmin1 = cmin
	  cxmax1 = cmin + (cmax - cmin) / 2.0
c
          ax2 = a0 
          ex2 = e0 
          ix2 = i0 
          ay2 = a 
          ey2 = e 
          iy2 = i 
c
	  cxmin2 = cxmax1 
	  cxmax2 = cmax
          linetest = .false.
ccc
c Line 3:
ccc
      elseif (c0min .eq. -1.0 .and. c0max .eq. 1.0) then
c
          ax1 = a 
          ex1 = e 
          ix1 = i 
          ay1 = a0 
          ey1 = e0 
          iy1 = i0 
	  cxmin1 = c0min
	  cxmax1 = c0min + (c0max - c0min) / 2.0
c
          ax2 = a 
          ex2 = e 
          ix2 = i 
          ay2 = a0 
          ey2 = e0 
          iy2 = i0 
c
	  cxmin2 = cxmax1 
	  cxmax2 = c0max
          linetest = .false.
ccc
c Line 4:
ccc
      elseif (c0min .eq. -1.0 .and. c0max .lt. 1.0 .and.
     $        cmin  .gt. -1.0 .and. cmax  .eq. 1.0) then 
c
          ax1 = a
          ex1 = e 
          ix1 = i 
          ay1 = a0 
          ey1 = e0 
          iy1 = i0 
	  cxmin1 = c0min
	  cxmax1 = c0min + (c0max - c0min) / 2.0
c
          ax2 = a0 
          ex2 = e0 
          ix2 = i0 
          ay2 = a 
          ey2 = e 
          iy2 = i 
ccc
c  Get a new value for C0min and C0max so you can integrate the
c  rest of the way.
ccc
          call c_equation (ax1, ay1, ex1, ey1, cxmin2, cxmax1)
	  cxmax2 = cmax
	  linetest = .true.
      else
	 write (6, *) 'The line did not make any of the checks'
      endif
c
      return
      end
      SUBROUTINE SKIP (IUNIT)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  ROUTINE TO SKIP LINES BEGINNING WITH THE STRING /*
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
      CHARACTER  STRING*80,LEADER*2
c
      DATA LEADER/'/*'/
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
   10 READ(IUNIT,1000,END=20) STRING
      call testchar (NCHAR, STRING)
      II = NCHAR
      IF(II.EQ.0) GOTO 10
c
      IF (INDEX(STRING,LEADER).EQ.1) THEN
         GOTO 10
      ELSE
         BACKSPACE (IUNIT)
      ENDIF
c
 1000 FORMAT(A80)
c
   20 RETURN
      END
C
C
      SUBROUTINE TESTCHAR (NCHAR, STRING)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
C  DETERMINES THE MINIMUM NONBLANK LENGTH OF A STRING
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
      CHARACTER*(*) STRING
      CHARACTER BLANK
c
      DATA BLANK/' '/
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
      NMAX=LEN(STRING)
      NCHAR=0
c
      DO 10 I=1,NMAX
         ITEST=NMAX-I+1
         IF(STRING(ITEST:ITEST).NE.BLANK) THEN
           NCHAR=ITEST
           RETURN
         ENDIF
10    CONTINUE
c
      RETURN
      END
